//转拼音插件缺少的文字，需自动转换
const PINYIN_LETTERS:any = {
    "酋": ["qiu"],
    "汶": ["wen"],
    "埔": ["pu"],
    "挝": ["wo"],
    "兹": ["zi"],
    "圭": ["gui"],
    "厄": ["e"],
    "韦": ["wei"],
    "茨": ["ci"],
    "喀": ["ka"],
    "迪": ["di"],
    "斐": ["fei"],
    "瑙": ["nao"],
    "梵": ["fan"],
    "陀": ["tuo"]
};

/**
 * 拼音列表转拼音首字母列表
 * @param {}} strList 
 */
function toFirstLetterList(strList:Array<string>) {
    let res = [];
    for (let i = 0; i < strList.length; i++) {
        let str = strList[i][0];
        res.push(str);
    }
    return res;
}


/**
 * 含有非中文保留非中文本身，否则转化为对应的拼音列表
 * type 默认0,0--全拼音;1--拼音首字母;2--全拼音和拼音首字母
 * 参数示例:
 * 1)入参:str--"ab测试",type--0;出参["abceshi"]
 * 2)入参首字母:str--"ab测试",type--1;出参["abcs"]
 * 3)多音字入参:str--"ab参数",type--0;出参["abcanshu","absenshu"]
 * 4)多音字入参和首字和:str--"ab参数",type--2;出参["abcs","abss"]
 */
function toPinYinList(str:string, type:number) {
    if (!str) {
        return "";
    }
    type = type ? type : 0;
    let pinyin = require("pinyin");
    let res = [];

    for (let i = 0; i < str.length; i++) {
        let letter = str[i];
        let pinyinList = [];
        let reg = /[\u4E00-\u9FA5]/g; //中文字符的正则
        if (reg.test(letter)) {
            //中文
            if (PINYIN_LETTERS[letter]) {
                pinyinList = PINYIN_LETTERS[letter]; //全拼音
            } else {
                pinyinList = pinyin(letter, {
                    style: pinyin.STYLE_NORMAL, // 设置拼音风格
                    heteronym: true
                })[0];
            }

            let firstLetterList = toFirstLetterList(pinyinList); //拼音首字母
            if (type == 1) {
                pinyinList = firstLetterList;
            } else if (type == 2) {
                pinyinList = pinyinList.concat(firstLetterList); //全拼音+拼音首字母
            }
        } else {
            //非中文
            pinyinList = [letter];
        }

        if (i == 0) {
            res = pinyinList;
        } else {
            let oldItemCount = res.length;
            let newRes:any = [];
            for (let m = 0; m < oldItemCount; m++) {
                for (let n = 0; n < pinyinList.length; n++) {
                    newRes.push(res[m].concat(pinyinList[n]));
                }
            }
            res = newRes;
        }
    }
    // console.log("toPinYinList", str, JSON.stringify(res));
    return res;
}



function deepCopy(obj:any):any {
    if (obj instanceof Array) {
        let n = [];
        for (let i = 0; i < obj.length; ++i) {
            n[i] = deepCopy(obj[i]);
        }
        return n;

    } else if (obj instanceof Object) {
        let n:any = {};
        for (let i in obj) {
            n[i] = deepCopy(obj[i]);
        }
        return n;
    } else {
        return obj;
    }
}

function copy(obj:any):any {
    if (obj instanceof Array) {
        let n = [];
        for (let i = 0; i < obj.length; ++i) {
            n[i] = obj[i];
        }
        return n;

    } else if (obj instanceof Object) {
        let n:any = {};
        for (let i in obj) {
            n[i] = obj[i];
        }
        return n;
    } else {
        return obj;
    }
}

function isString(obj:any):Boolean {
    if (obj === null) {
        return false;
    }
    return Object.prototype.toString.call(obj) === "[object String]";
}


const ElementUtil = {
    getScrollTop: function() {
        let scrollTop = document.documentElement.scrollTop || document.body.scrollTop;
        return scrollTop;
    },

    getScrollLeft: function() {
        let scrollLeft = document.documentElement.scrollLeft || document.body.scrollLeft;
        return scrollLeft;
    }
};

const ObjectUtil = {
    /**
     * 深拷贝
     */
    deepCopy: function(obj:any):any {
        // 若不是对象类型或是null类型，直接输出
        if (typeof obj !== 'object' || obj === null) {
            return obj;
        }
        return deepCopy(obj);
    },

    /**
     * 浅拷贝
     */
    copy: function(obj:any):any {
        // 若不是对象类型或是null类型，直接输出
        if (typeof obj !== 'object' || obj === null) {
            return obj;
        }
        return copy(obj);
    },

    /**
     * 从sourceObj浅拷贝到targetObj
     */
    copyTo: function(sourceObj:any, targetObj:any):any {
        // 若不是对象类型或是null类型，直接输出
        if (typeof sourceObj !== 'object' || sourceObj === null) {
            return;
        }
        if (typeof targetObj !== 'object' || targetObj === null) {
            return;
        }

        for (let key in sourceObj) {
            targetObj[key] = sourceObj[key];
        }
        return targetObj;
    }

};

const ArrayUtil = {
    /**
     * 若数组项为object，返回由每个数组项对应key的value值组成的新数组
     */
    getListByKey: function(arr:Array<any>, key:string) {
        let res:any = [];
        if (arr && arr instanceof Array) {
            arr.forEach(ele => {
                if (ele && ele.hasOwnProperty(key)) {
                    res.push(ele[key]);
                }
            });
        } else {
            // console.log("common.ArrayUtil getListByKey：there is an error about params ", arr)
        }
        return res;
    },

    /**
     * 数组去重,直接修改原数组
     */
    distinct: function(arr:any) {
        let len = arr.length;
        for (let i = len - 1; i >= 0; i--) {
            let curValue = arr[i];
            if (arr.indexOf(curValue) != i) {
                arr.splice(i, 1);
            }
        }
    }
};

const StringUtil = {
    /**
     * 去除字符串收尾空字符
     */
    trim: function(str:string) {
        if (typeof(str) == "string") {
            return str.replace(/^\s+|\s+$/gm, '');
        }
    },

    /**
     * 判断对象是否是字符串
     * @param {*} obj 
     */
    isString(obj:any) {
        return isString(obj);
    },

    /**
     * 汉子转拼音
     */
    toPinYinList: toPinYinList,

    /**
     * 检查字符串是否为小数
     */
    isFloatNumber: function(str:string) {
        if (!isString(str)) {
            return false;
        }
        let reg = /^\d+(\.\d+)?$/;
        if (reg.test(str)) {
            return true;
        }
        return false;
    },

    /**
     * 检查字符串是否为纯数字
     */
    isIntNumber: function(str:string) {
        if (!isString(str)) {
            return false;
        }
        let reg = /^[0-9]+?[0-9]*$/;
        if (reg.test(str)) {
            return true;
        }
        return false;
    }
};

const common = {
    ElementUtil: ElementUtil,
    ObjectUtil: ObjectUtil,
    ArrayUtil: ArrayUtil,
    StringUtil: StringUtil,
    rmbUpperCase: rmbUpperCase


};


/** 
 * @param {金额转大写 -}
 * @param {金额 -} n 
 */
function rmbUpperCase(n) {
    var fraction = ['角', '分'];
    var digit = ['零', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖'];
    var unit = [['元', '万', '亿'], ['', '拾', '佰', '仟']];
    var head = n < 0 ? '欠' : '';
    n = Math.abs(n);

    var s = '';

    for (var i = 0; i < fraction.length; i++) {
        s += (digit[Math.floor(n * 10 * Math.pow(10, i)) % 10] + fraction[i]).replace(/零./, '');
    }
    s = s || '整';
    n = Math.floor(n);

    for (var i = 0; i < unit[0].length && n > 0; i++) {
        var p = '';
        for (var j = 0; j < unit[1].length && n > 0; j++) {
            p = digit[n % 10] + unit[1][j] + p;
            n = Math.floor(n / 10);
        }
        s = p.replace(/(零.)*零$/, '').replace(/^$/, '零') + unit[0][i] + s;
    }
    return head + s.replace(/(零.)*零元/, '元').replace(/(零.)+/g, '零').replace(/^整$/, '零元整');
}


export {
    common
}