import Theme from "./Theme";

let curTheme = "";
const DEFAULT_THEME: string = "default";

const THEME_MAP: any = {
  default: new Theme("default-theme", "default-theme", ""),
  yellow: new Theme("test-yellow-theme", "test-yellow-theme", ""),
  openPlatform: new Theme("openplatform-theme", "openplatform-theme", "")
};

setTheme(DEFAULT_THEME);

/**
 * 获取全部主题
 */
function getThemes(): any {
  return Object.values(THEME_MAP);
}

/**
 * 获取全部主题
 */
function getThemeNames(): any {
  return Object.keys(THEME_MAP);
}

/**
 * 获取当前主题
 */
function getCurTheme() {
  return curTheme;
}

/**
 * 设置主题
 * @param {*} newTheme
 */
function setTheme(newTheme: string = "") {
  // debugger

  newTheme = newTheme || DEFAULT_THEME;
  if (!newTheme || curTheme == newTheme) {
    return;
  }
  let oldThemeItem: Theme = THEME_MAP[curTheme];
  let themeItem: Theme = THEME_MAP[newTheme];
  if (!themeItem) {
    themeItem = THEME_MAP[DEFAULT_THEME];
  }
  if (oldThemeItem) {
    document.body.classList.remove(oldThemeItem.className);
  }
  let className: string = themeItem.className;
  curTheme = newTheme;
  document.body.classList.add(className);
}

const ThemeManager = {
  setTheme: setTheme,
  getCurTheme: getCurTheme,
  getThemes: getThemes,
  getThemeNames:getThemeNames
};

export { ThemeManager };
